import java.awt.*;
import java.awt.image.*;
import java.util.*;

/**
 * Region growing algorithm: finds and holds regions in an image.
 * Each region is a list of contiguous points with colors similar to a target color.
 * Scaffold for PS-1, Dartmouth CS 10, Fall 2016
 * 
 * @author Chris Bailey-Kellogg, Winter 2014 (based on a very different structure from Fall 2012)
 * @author Travis W. Peters, Dartmouth CS 10, Updated Winter 2015
 * @author CBK, Spring 2015, updated for CamPaint
 * @author Logan Tucker, Winter 2022
 */
public class RegionFinder {
	private static final int maxColorDiff = 20;				// how similar a pixel color must be to the target color, to belong to a region
	private static final int minRegion = 50; 				// how many points in a region to be worth considering

	private BufferedImage image;                            // the image in which to find regions
	private BufferedImage recoloredImage;                   // the image with identified regions recolored

	public ArrayList<ArrayList<Point>> regions;			    // a region is a list of points
															// so the identified regions are in a list of lists of points

	public RegionFinder() {
		this.image = null;
	}

	public RegionFinder(BufferedImage image) {
		this.image = image;		
	}

	public void setImage(BufferedImage image) {
		this.image = image;
	}

	public BufferedImage getImage() {
		return image;
	}

	public BufferedImage getRecoloredImage() {
		return recoloredImage;
	}

	/**
	 * Sets regions to the flood-fill regions in the image, similar enough to the trackColor.
	 */
	public void findRegions(Color targetColor) {
		// TODO: YOUR CODE HERE

		regions = new ArrayList<ArrayList<Point>>();
		BufferedImage visited = new BufferedImage(image.getWidth(), image.getHeight(), BufferedImage.TYPE_INT_ARGB);

		// Loop over each pixel in the image
		for (int y = 0; y < image.getHeight(); y++) {
			for (int x = 0; x < image.getWidth(); x++) {


				Color c = new Color(image.getRGB(x,y));
				Point simPixel = new Point(x, y);

				// determine weather the pixel is desirable (of the desired color and is being looked at for the first time)
				if (colorMatch(c, targetColor) && visited.getRGB(simPixel.x, simPixel.y) == 0) {

					// start a new Region to hold pixels
					ArrayList<Point> region = new ArrayList<Point>();

					// make a list of all pixels worth considering for inclusion in the current region
					ArrayList<Point> toVisit = new ArrayList<Point>();
					toVisit.add(simPixel);

					// as long as there are pixels worth considering (for inclusion in the current region), inspect them and their neighbors
					while (toVisit.size() > 0) {
						Point p = toVisit.remove(toVisit.size()-1);

						// if a pixel worth considering has not been visited make it part of a region (its desirable)
						if(visited.getRGB(p.x, p.y) == 0) {
							region.add(p);

							// mark it as visited as not to add it to a region twice
							visited.setRGB(p.x, p.y, 1);

							// consider this pixel's neighbors (that are within the image)
							for (int ny = Math.max(0, p.y-1); ny <= Math.min((p.y+1), (image.getHeight()-1)); ny++) {
								for (int nx = Math.max(0, p.x-1); nx <= Math.min((p.x+1), (image.getWidth()-1)); nx++) {
									Color neighborColor = new Color(image.getRGB(nx,ny));
									Point neighborPixel = new Point(nx, ny);

									// determine weather the pixel is desirable
									if (colorMatch(neighborColor, targetColor) && visited.getRGB(neighborPixel.x, neighborPixel.y) == 0) {

										// label it as worth considering
										toVisit.add(neighborPixel);
									}
								}
							}
						}

					}
					// If the region is big enough to be worth keeping, do so
					if (region.size() >= minRegion) {
						regions.add(region);
					}
				}
			}
		}
	}


	/**
	 * Tests whether the two colors are "similar enough" (your definition, subject to the maxColorDiff threshold, which you can vary).
	 */
	private static boolean colorMatch(Color c1, Color c2) {
		// TODO: YOUR CODE HERE

		// get absolute value of difference between target color RGBs and prospect color RGBs
		int rdiff = Math.abs(c1.getRed() - c2.getRed());
		int gdiff = Math.abs(c1.getGreen() - c2.getGreen());
		int bdiff = Math.abs(c1.getBlue() - c2.getBlue());

		return (rdiff <= maxColorDiff && gdiff <= maxColorDiff && bdiff <= maxColorDiff);
	}

	/**
	 * Returns the largest region detected (if any region has been detected)
	 */
	public ArrayList<Point> largestRegion() {
		// TODO: YOUR CODE HERE

		int largestSize = 0;
		int largestIndex = 0;

		for (int i = 0; i < regions.size(); i++) {
			if (regions.get(i).size() > largestSize) {
				largestSize = regions.get(i).size();
				largestIndex = i;
			}
		}

		return (regions.get(largestIndex));
	}

	/**
	 * Sets recoloredImage to be a copy of image, 
	 * but with each region a uniform random color, 
	 * so we can see where they are
	 */
	public void recolorImage() {
		// First copy the original
		recoloredImage = new BufferedImage(image.getColorModel(), image.copyData(null), image.getColorModel().isAlphaPremultiplied(), null);
		// Now recolor the regions in it
		// TODO: YOUR CODE HERE

		for (ArrayList<Point> region: regions) {
			// create random color
			int c = (int) (16777216 * Math.random());

			for (Point point: region) {
				// paint recolored image pixel at coordinates (x, y) random color
				recoloredImage.setRGB(point.x, point.y, c);
			}
		}
	}
}
